/*Network monitor for system tray*/
/*(c) Copyright Barry Kauler 2010, bkhome.org*/
/*GPL license: /usr/share/doc/legal/gpl-2.0.txt*/
/*first release, version 2.0, may 18, 2010*/
/*version 2.1: new icons*/
/*version 2.3: icons now external, now displays dead icon correctly*/
/*version 2.3.2: still trying to get icon to update correctly*/
/*version 2.4: display tx/rx in KB if <1024, improved mouse-click operations*/
/*version 2.4.1: fix icon update when disconnect/reconnect*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>
#include <dirent.h>
#include <errno.h>

#include "networkblank.xpm"
#include "networkboth.xpm"
#include "networkdead.xpm"
#include "networkin.xpm"
#include "networkout.xpm"

#include "networkboth-dialup.xpm"
#include "networkin-dialup.xpm"
#include "networkout-dialup.xpm"
#include "networkblank-dialup.xpm"

#include "networkboth-wireless.xpm"
#include "networkin-wireless.xpm"
#include "networkout-wireless.xpm"
#include "networkblank-wireless.xpm"

GdkPixbuf *networkblank_pixbuf;
GdkPixbuf *networkboth_pixbuf;
GdkPixbuf *networkdead_pixbuf;
GdkPixbuf *networkin_pixbuf;
GdkPixbuf *networkout_pixbuf;

GdkPixbuf *networkblank_wireless_pixbuf;
GdkPixbuf *networkin_wireless_pixbuf;
GdkPixbuf *networkout_wireless_pixbuf;
GdkPixbuf *networkboth_wireless_pixbuf;

GdkPixbuf *networkin_dialup_pixbuf;
GdkPixbuf *networkout_dialup_pixbuf;
GdkPixbuf *networkboth_dialup_pixbuf;
GdkPixbuf *networkblank_dialup_pixbuf;

GtkWidget *pref_image;
GtkWidget *enable_image;
GtkWidget *disable_image;
GtkWidget *info_image;

GtkStatusIcon *tray_icon;
unsigned int interval = 500; /*update interval in milliseconds*/

char *netdir="/sys/class/net";
DIR *dip;
struct dirent *dit;
//int  i = 0;
char buf1[1024];
FILE *fp;
char char1;
int flagactive=0;
int flagactiveprev=0;
char infomsg[256];
char rxstr[52];
long int rxacc=0;
char rxaccstr[64];
char txstr[52];
long int txacc=0;
char txaccstr[64];
long int rxaccprev=0;
long int txaccprev=0;
int flagtransfer=0;
int flagtransferprev=0;
int loopcnt=0;
int breakcnt=0;
int flagdisconnect=0;
int wireless=-1;
int eth=-1;
int dialup=0;


GError *gerror;


void wirelessup(){
	wireless=1;
}
void wirelessdown(){
	wireless=0;
}
void ethdown(){
	eth=0;
}
void ethup(){
	eth=1;
}


gboolean Update(gpointer ptr);
int find_active();

int find_active() {

    flagactive=0;
    infomsg[0]=0;
    strcat(infomsg,"Frisbee network interfaces: ");
    
    dip=opendir(netdir);
    if (dip==NULL) return 1;
    dialup=0;
    
    while ((dit = readdir(dip)) != NULL)
    {
        if (strcmp(dit->d_name,".")!=0 && strcmp(dit->d_name,"..")!=0 && strcmp(dit->d_name,"lo")!=0  ){
			
			if ( strncmp(dit->d_name,"ppp",3)==0 ){
				dialup=1;
			}
			
            buf1[0]=0;
            strcat(buf1,netdir);
            strcat(buf1,"/");
            strcat(buf1,dit->d_name);
            strcat(buf1,"/dormant");
            fp=fopen(buf1,"r");
            if (fp!=NULL) {
                char1=fgetc(fp);
                fclose(fp);
                //if interface active, char1==0...
                if (char1=='0') {
                    flagactive=flagactive+1;
                    strcat(infomsg,dit->d_name);
                    strcat(infomsg," ");
                    //accumulate bytes received...
                    buf1[0]=0;
                    strcat(buf1,netdir);
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/rx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(rxstr,12,fp);
                        rxacc=atol(rxstr)/1024;
                        fclose(fp);
                    }
                    //accumulate bytes transmitted...
                    buf1[0]=0;
                    strcat(buf1,netdir);
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/tx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(txstr,12,fp);
                        txacc=atol(txstr)/1024;
                        fclose(fp);
                    }
                    
                }
            }
        }
    }

    closedir(dip);
    if (flagactive==0 ) {
        strcat(infomsg,"none ");
        strcat(infomsg," Click this icon for network setup");
    }
    else {
        if (rxacc < 1024) {
            sprintf(rxaccstr,"%.2f",((float)rxacc));
            strcat(infomsg," Rx:");
            strcat(infomsg,rxaccstr);
            strcat(infomsg,"KB ");
        }
        else {
            sprintf(rxaccstr,"%.2f",((float)rxacc/1024.0));
            strcat(infomsg," Rx:");
            strcat(infomsg,rxaccstr);
            strcat(infomsg,"MB ");
        }
        if (txacc < 1024) {
            sprintf(txaccstr,"%.2f",((float)txacc));
            strcat(infomsg," Tx:");
            strcat(infomsg,txaccstr);
            strcat(infomsg,"KB");
        }
        else {
            sprintf(txaccstr,"%.2f",((float)txacc/1024.0));
            strcat(infomsg," Tx:");
            strcat(infomsg,txaccstr);
            strcat(infomsg,"MB");
        }
    }
}

gboolean Update(gpointer ptr) {

    find_active();
    loopcnt=loopcnt+1;
    
    if (flagactive==0 ) flagtransfer=0;
    else {
        flagtransfer=1;
        if (rxacc!=rxaccprev) flagtransfer=2;
        if (txacc!=txaccprev) flagtransfer=3;
        if (txacc!=txaccprev && rxacc!=rxaccprev) flagtransfer=4;
    }
    //printf("loopcnt=%i flagtransfer=%i flagactive=%i breakcnt=%i\n",loopcnt,flagtransfer,flagactive,breakcnt); fflush(stdout); //TEST
    //lot of trouble with this logic, so introduce breakcnt to force icon update if stuck...
    if (flagtransferprev == flagtransfer) {
    	if (loopcnt != 1) { //want to update icon on first loop.
    	    if (flagactive == flagactiveprev && flagtransfer == flagtransferprev && flagtransfer == 1) breakcnt=breakcnt+1; //no change.
    	    if (flagactive == flagactiveprev && flagtransfer == flagtransferprev ) breakcnt=breakcnt+1; //no change.
		}
		else breakcnt=0;
		if (breakcnt != 0 && breakcnt < 8) return; //force update after 4 seconds.
    }
    breakcnt=0;
    flagactiveprev=flagactive;
    txaccprev=txacc;
    rxaccprev=rxacc;
    flagtransferprev=flagtransfer;
    
    //printf("dialup=%i wireless=%i eth=%i\n",dialup,wireless,eth); fflush(stdout); //TEST
	if ( dialup == 1 ) {
	    if (flagtransfer==0) gtk_status_icon_set_from_pixbuf(tray_icon,networkdead_pixbuf);
	    else if (flagtransfer==1) gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_dialup_pixbuf);
	    else if (flagtransfer==2) gtk_status_icon_set_from_pixbuf(tray_icon,networkin_dialup_pixbuf);
	    else if (flagtransfer==3) gtk_status_icon_set_from_pixbuf(tray_icon,networkout_dialup_pixbuf);
	    else if (flagtransfer==4) gtk_status_icon_set_from_pixbuf(tray_icon,networkboth_dialup_pixbuf);
	}
       else if (wireless == 1){
	    if (flagtransfer==0) gtk_status_icon_set_from_pixbuf(tray_icon,networkdead_pixbuf);
	    else if (flagtransfer==1) gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_wireless_pixbuf);
	    else if (flagtransfer==2) gtk_status_icon_set_from_pixbuf(tray_icon,networkin_wireless_pixbuf);
	    else if (flagtransfer==3) gtk_status_icon_set_from_pixbuf(tray_icon,networkout_wireless_pixbuf);
	    else if (flagtransfer==4) gtk_status_icon_set_from_pixbuf(tray_icon,networkboth_wireless_pixbuf);
	 }
	else if ( eth == 1 ) {
	    if (flagtransfer==0) gtk_status_icon_set_from_pixbuf(tray_icon,networkdead_pixbuf);
	    else if (flagtransfer==1) gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_pixbuf);
	    else if (flagtransfer==2) gtk_status_icon_set_from_pixbuf(tray_icon,networkin_pixbuf);
	    else if (flagtransfer==3) gtk_status_icon_set_from_pixbuf(tray_icon,networkout_pixbuf);
	    else if (flagtransfer==4) gtk_status_icon_set_from_pixbuf(tray_icon,networkboth_pixbuf);
	}
	else{
		gtk_status_icon_set_from_pixbuf(tray_icon,networkdead_pixbuf);
	}
		
    
    //update tooltip...
    gtk_status_icon_set_tooltip(tray_icon, infomsg);
}

void  view_popup_menu_onSetupNetworking (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onSetupNetworking (GtkWidget *menuitem, gpointer userdata)
  {
    /* we passed the view as userdata when we connected the signal */
    /*GtkTreeView *treeview = GTK_TREE_VIEW(userdata);*/
    system("Frisbee & ");
  }

void  view_popup_menu_onNetworkStatus (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onNetworkStatus (GtkWidget *menuitem, gpointer userdata)
  {
    system("/usr/local/Frisbee/ipinfo & ");
  }

void  view_popup_menu_onDisconnect (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onDisconnect (GtkWidget *menuitem, gpointer userdata)
  {
    system("/usr/local/Frisbee/disconnect&");
  }

void  view_popup_menu_onReconnect (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onReconnect (GtkWidget *menuitem, gpointer userdata)
{
    system("/usr/local/Frisbee/connect&");
}

void tray_icon_on_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
        GtkWidget *menu, *menuitem;
	pref_image=gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
	disable_image=gtk_image_new_from_stock(GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_MENU);
	enable_image=gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
	info_image=gtk_image_new_from_stock(GTK_STOCK_INFO, GTK_ICON_SIZE_MENU);

        menu = gtk_menu_new();

        menuitem = gtk_image_menu_item_new_with_label("Setup networking");
	gtk_image_menu_item_set_image(menuitem,pref_image);
        g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onSetupNetworking, status_icon);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        menuitem = gtk_image_menu_item_new_with_label("Network status information");
	gtk_image_menu_item_set_image(menuitem,info_image);
        g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onNetworkStatus, status_icon);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
        if ( wireless == 1) {
            menuitem = gtk_image_menu_item_new_with_label("Disable wireless network");
            gtk_image_menu_item_set_image(menuitem,disable_image);
            g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onDisconnect, status_icon);
            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
        }
        if (wireless == 0 ) {
            menuitem = gtk_image_menu_item_new_with_label("Enable wireless network");
            gtk_image_menu_item_set_image(menuitem,enable_image);
            g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onReconnect, status_icon);
            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
        }
        gtk_widget_show_all(menu);
        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
}

void tray_icon_on_click(GtkStatusIcon *status_icon, gpointer user_data)
{
    system("Frisbee&");
}

static GtkStatusIcon *create_tray_icon() {

    tray_icon = gtk_status_icon_new();
    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(tray_icon_on_menu), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(tray_icon_on_menu), NULL);

	networkblank_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkblank_xpm);
    networkboth_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkboth_xpm);
    networkdead_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkdead_xpm);
    networkin_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkin_xpm);
    networkout_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkout_xpm);
    
    networkboth_wireless_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkboth_wireless_xpm);
    networkin_wireless_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkin_wireless_xpm);
    networkout_wireless_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkout_wireless_xpm);
    networkblank_wireless_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkblank_wireless_xpm);
    
    networkboth_dialup_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkboth_dialup_xpm);
    networkin_dialup_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkin_dialup_xpm);
    networkout_dialup_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkout_dialup_xpm);
    networkblank_dialup_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkblank_dialup_xpm);

    gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_pixbuf);

    gtk_status_icon_set_tooltip(tray_icon, "No active network interfaces");
    gtk_status_icon_set_visible(tray_icon, TRUE);
    return tray_icon;
}

int main(int argc, char **argv) {
	
    signal(SIGALRM, wirelessdown);
    signal(SIGCONT, wirelessup);
    signal(SIGUSR1, ethup);
    signal(SIGUSR2, ethdown);
	
    gtk_init(&argc, &argv);
    tray_icon = create_tray_icon();
        
    gtk_timeout_add(interval, Update, NULL);

    gtk_main();
    return 0;
}
